# generated by datamodel-codegen:
#   filename:  openapi.json

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Literal, Optional, Union

from pydantic import BaseModel, Extra, Field


class LoincCodeFilter(BaseModel):
    class Config:
        extra = Extra.forbid

    loincCodes: List[str]


class TrainingApproachBase(BaseModel):
    class Config:
        extra = Extra.forbid

    id: str
    """
    UUID uniquely identifying this training approach.
    """
    name: str
    description: str


class TuningJobTrainingApproach(TrainingApproachBase):
    type: Literal["tuningJob"]
    tuningJobConfig: Dict[str, Any]
    """
    TODO - stub out this type.
    """
    deployConfig: Dict[str, Any]
    """
    TODO - stub out this type.
    """


class TrainingApproach(BaseModel):
    __root__: TuningJobTrainingApproach


class Split(BaseModel):
    class Config:
        extra = Extra.forbid

    n: float
    """
    The number of examples in this split.
    """
    start: float
    """
    Timestamp marking the beginning of the data for this split (inclusive). Expressed as milliseconds since the UTC epoch.
    """
    end: float
    """
    Timestamp marking the end of the data for this split (exclusive). Expressed as milliseconds since the UTC epoch.
    """
    uri: str = Field(..., regex="^s3:\\/\\/.+$")
    """
    S3 URI of where a copy of this exact split is saved.
    """


class Metric(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    """
    The name of the metric e.g. "Cross Entopy Loss", "Accuracy", "F1 Macro", etc.
    """
    value: float


class ApprovalChoice(Enum):
    approved = "approved"
    rejected = "rejected"


class ApprovalDecisionBase(BaseModel):
    class Config:
        extra = Extra.forbid

    timestamp: float
    """
    Timestamp of when the decision was made. Expressed as milliseconds since the UTC epoch.
    """
    decision: ApprovalChoice


class SystemApprovalDecision(ApprovalDecisionBase):
    actor: Literal["system"]


class UserApprovalDecision(ApprovalDecisionBase):
    actor: Literal["user"]
    user: str


class ApprovalDecision(BaseModel):
    __root__: Union[SystemApprovalDecision, UserApprovalDecision]


class RunSplits(BaseModel):
    """
    Information about the different splits of the dataset used to train this model version.
    """

    class Config:
        extra = Extra.forbid

    train: Split
    val: Split
    test: Split


class RunMetrics(BaseModel):
    class Config:
        extra = Extra.forbid

    challenger: List[Metric]
    """
    Metrics about how the model version trained in this run (the challenger) performed on this run's test set.
    """
    champion: Optional[List[Metric]] = None
    """
    Metrics about how the current best model (the champion) performed on this run's test set.
    """


class ImageSegmentationProblem(BaseModel):
    class Config:
        extra = Extra.forbid

    problemType: Literal["imgSeg"]
    imageFilter: LoincCodeFilter
    """
    Used to filter through patient data to identify the input images for the model.
    """
    maskFilter: LoincCodeFilter
    """
    Used to filter through patient data to identify the image segmentation mask training label images for the model.
    """


class ImageClassificationProblem(BaseModel):
    class Config:
        extra = Extra.forbid

    problemType: Literal["imgClf"]
    imageFilter: LoincCodeFilter
    """
    Used to filter through patient data to identify the input images for the model.
    """


class MlProblemDefinition(BaseModel):
    __root__: Union[ImageSegmentationProblem, ImageClassificationProblem]


class ModelConfigBase(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    description: str
    problemDefinition: MlProblemDefinition
    avgLatencyUb: Optional[float] = None
    """
    The allowed upper bound on the model's average latency, in seconds. A lower upper bound will mean higher inference costs.
    """


class ModelConfig(ModelConfigBase):
    id: str
    """
    UUID uniquely identifying this model config.
    """
    accountId: str
    trainingApproach: TrainingApproach
    deployedId: Optional[str] = None
    """
    The ID of the model version currently deployed for this model config.
    """


class ModelRun(BaseModel):
    """
    TODO - add the hyperparameters of the model artifact to this object.
    """

    class Config:
        extra = Extra.forbid

    id: str
    """
    UUID uniquely identifying this model run.
    """
    slug: str
    """
    Human-readable slug that acts as an auto-generated name for the run. This field is not guaranteed to be unique, so cannot be used reliably as a unique identifier.
    """
    modelId: str
    """
    ID of the model config this run was produced for.
    """
    accountId: str
    modelArtifactUri: Optional[str] = None
    """
    S3 URI location where the final trained model artifact produced by this run is saved to.
    """
    status: Literal["running", "succeeded", "failed"]
    error: Optional[str] = None
    """
    If `status` is 'failed', gives the reason why the run failed.
    """
    start: float
    """
    Timestamp of when the run started. Expressed as milliseconds since the UTC epoch.
    """
    end: Optional[float] = None
    """
    Timestamp of when the run ended. Expressed as milliseconds since the UTC epoch.
    """
    splits: Optional[RunSplits] = None
    metrics: RunMetrics
    approvals: List[ApprovalDecision]
    """
    Decisions made by various actors representing whether they think this model version should be used in production and become the new champion.
    """
    problemDefinition: MlProblemDefinition
    trainingApproach: TrainingApproach
